/**
 * Copyright (c) 2017-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Abel Hegedus (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.persistence.emf.transform;

import com.google.common.collect.Iterables;
import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.diffmerge.patch.AbstractModelPatchEntry;
import org.eclipse.emf.diffmerge.patch.AttributeEntry;
import org.eclipse.emf.diffmerge.patch.ElementEntry;
import org.eclipse.emf.diffmerge.patch.ReferenceEntry;
import org.eclipse.emf.diffmerge.patch.StructuralFeatureEntry;
import org.eclipse.emf.diffmerge.patch.api.BaseModelPatchMetadata;
import org.eclipse.emf.diffmerge.patch.api.ChangeDirection;
import org.eclipse.emf.diffmerge.patch.api.Identifiable;
import org.eclipse.emf.diffmerge.patch.api.ModelPatch;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchEntry;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EAttributeEntry;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EChangeDirection;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EElementEntry;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EIdentifiable;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EModelPatch;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EModelPatchEMFMetadata;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EModelPatchEntry;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EModelPatchMetadata;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EReferenceEntry;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EStructuralFeatureEntry;
import org.eclipse.emf.diffmerge.patch.runtime.EMFModelPatchMetadata;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class EModelPatchToPOJO {
  private final EModelPatch source;
  
  private final ModelPatch target;
  
  private final Logger logger;
  
  public EModelPatchToPOJO(final EModelPatch source, final ModelPatch target, final Logger logger) {
    this.source = source;
    this.target = target;
    this.logger = logger;
  }
  
  public void transform() {
    this.transformElement(this.source.getMetadata());
    final Consumer<EModelPatchEntry> _function = (EModelPatchEntry it) -> {
      this.transformElement(it);
    };
    Iterables.<EModelPatchEntry>filter(this.source.getEntries(), EModelPatchEntry.class).forEach(_function);
  }
  
  protected void transformElement(final EModelPatchMetadata emetadata) {
    BaseModelPatchMetadata _xifexpression = null;
    if ((emetadata instanceof EModelPatchEMFMetadata)) {
      EMFModelPatchMetadata _eMFModelPatchMetadata = new EMFModelPatchMetadata();
      final Procedure1<EMFModelPatchMetadata> _function = (EMFModelPatchMetadata it) -> {
        List<String> _modelUriList = it.getModelUriList();
        EList<String> _modelUris = ((EModelPatchEMFMetadata)emetadata).getModelUris();
        Iterables.<String>addAll(_modelUriList, _modelUris);
        List<String> _usedNamespaceUris = it.getUsedNamespaceUris();
        EList<String> _usedNamespaceUris_1 = ((EModelPatchEMFMetadata)emetadata).getUsedNamespaceUris();
        Iterables.<String>addAll(_usedNamespaceUris, _usedNamespaceUris_1);
      };
      _xifexpression = ObjectExtensions.<EMFModelPatchMetadata>operator_doubleArrow(_eMFModelPatchMetadata, _function);
    } else {
      _xifexpression = new BaseModelPatchMetadata();
    }
    final BaseModelPatchMetadata metadata = _xifexpression;
    final Procedure1<BaseModelPatchMetadata> _function_1 = (BaseModelPatchMetadata it) -> {
      it.setAuthor(emetadata.getAuthor());
      it.setCreatedAt(emetadata.getCreatedAt());
      it.setDescription(emetadata.getDescription());
      it.setInput(emetadata.getInput());
    };
    ObjectExtensions.<BaseModelPatchMetadata>operator_doubleArrow(metadata, _function_1);
    this.target.setMetadata(metadata);
    return;
  }
  
  protected void transformElement(final EModelPatchEntry entry) {
    final ModelPatchEntry eentry = this.transformEntry(entry);
    this.target.getEntries().add(eentry);
    return;
  }
  
  protected ModelPatchEntry _transformEntry(final EElementEntry entry) {
    ElementEntry _elementEntry = new ElementEntry();
    final Procedure1<ElementEntry> _function = (ElementEntry it) -> {
      this.transformAbstractEntryFeatures(entry, it);
      it.setType(this.toIdentifiable(entry.getType()));
    };
    return ObjectExtensions.<ElementEntry>operator_doubleArrow(_elementEntry, _function);
  }
  
  protected ModelPatchEntry _transformEntry(final EAttributeEntry entry) {
    AttributeEntry _attributeEntry = new AttributeEntry();
    final Procedure1<AttributeEntry> _function = (AttributeEntry it) -> {
      this.transformStructuralFeatureEntryFeatures(entry, it);
      it.setValue(entry.getValue());
    };
    return ObjectExtensions.<AttributeEntry>operator_doubleArrow(_attributeEntry, _function);
  }
  
  protected ModelPatchEntry _transformEntry(final EReferenceEntry entry) {
    ReferenceEntry _referenceEntry = new ReferenceEntry();
    final Procedure1<ReferenceEntry> _function = (ReferenceEntry it) -> {
      this.transformStructuralFeatureEntryFeatures(entry, it);
      it.setTarget(this.toIdentifiable(entry.getTarget()));
    };
    return ObjectExtensions.<ReferenceEntry>operator_doubleArrow(_referenceEntry, _function);
  }
  
  protected void transformAbstractEntryFeatures(final EModelPatchEntry eentry, final AbstractModelPatchEntry entry) {
    entry.setDirection(this.transformDirection(eentry.getDirection()));
    entry.setContext(this.toIdentifiable(eentry.getContext()));
  }
  
  protected void transformStructuralFeatureEntryFeatures(final EStructuralFeatureEntry eentry, final StructuralFeatureEntry entry) {
    this.transformAbstractEntryFeatures(eentry, entry);
    entry.setFeature(this.toIdentifiable(eentry.getFeature()));
  }
  
  protected Identifiable toIdentifiable(final EIdentifiable eidentifiable) {
    String _targetIdentifier = eidentifiable.getTargetIdentifier();
    return new Identifiable(_targetIdentifier);
  }
  
  protected ChangeDirection transformDirection(final EChangeDirection direction) {
    ChangeDirection _switchResult = null;
    if (direction != null) {
      switch (direction) {
        case ADD:
          _switchResult = ChangeDirection.ADD;
          break;
        case REMOVE:
          _switchResult = ChangeDirection.REMOVE;
          break;
        default:
          break;
      }
    }
    return _switchResult;
  }
  
  protected ModelPatchEntry transformEntry(final EModelPatchEntry entry) {
    if (entry instanceof EAttributeEntry) {
      return _transformEntry((EAttributeEntry)entry);
    } else if (entry instanceof EReferenceEntry) {
      return _transformEntry((EReferenceEntry)entry);
    } else if (entry instanceof EElementEntry) {
      return _transformEntry((EElementEntry)entry);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(entry).toString());
    }
  }
}
