/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Balazs Grill, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.runtime.modelaccess;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.diffmerge.patch.runtime.modelaccess.AbstractEMFModleAccess;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.viatra.transformation.runtime.emf.modelmanipulation.IModelManipulations;
import org.eclipse.xtext.xbase.lib.Exceptions;

@SuppressWarnings("all")
public class ViatraModelAccess extends AbstractEMFModleAccess {
  private final IModelManipulations viatraModelManipulator;
  
  private final Resource temporaryResource;
  
  public ViatraModelAccess(final IModelManipulations viatraModelManipulator, final Resource temporaryResource) {
    this.viatraModelManipulator = viatraModelManipulator;
    this.temporaryResource = temporaryResource;
  }
  
  @Override
  protected EObject doCreate(final EClass type) {
    try {
      return this.viatraModelManipulator.create(this.temporaryResource, type);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  protected void doRemove(final EObject element) {
    try {
      this.viatraModelManipulator.remove(element);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  protected void doAdd(final EObject container, final EStructuralFeature feature, final Object element) {
    try {
      boolean _isMany = feature.isMany();
      if (_isMany) {
        if (((feature instanceof EReference) && ((EReference) feature).isContainment())) {
          if ((element instanceof EObject)) {
            this.viatraModelManipulator.moveTo(((EObject)element), container, ((EReference) feature));
          } else {
            throw new IllegalArgumentException("Only EObjects can be added to EReferences.");
          }
        } else {
          this.viatraModelManipulator.add(container, feature, element);
        }
      } else {
        this.viatraModelManipulator.set(container, feature, element);
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  protected void doAdd(final EObject container, final EStructuralFeature feature, final Object element, final int index) {
    try {
      if (((feature instanceof EReference) && ((EReference) feature).isContainment())) {
        if ((element instanceof EObject)) {
          this.viatraModelManipulator.moveTo(((EObject) element), container, ((EReference) feature), index);
        } else {
          throw new IllegalArgumentException("Only EObjects can be added to EReferences.");
        }
      } else {
        this.viatraModelManipulator.add(container, feature, element, index);
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  protected void doRemove(final EObject container, final EStructuralFeature feature, final Object element) {
    try {
      boolean _isMany = feature.isMany();
      if (_isMany) {
        this.viatraModelManipulator.remove(container, feature, element);
      } else {
        this.viatraModelManipulator.set(container, feature, feature.getDefaultValue());
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  protected void doRemove(final EObject container, final EStructuralFeature feature, final int index) {
    try {
      this.viatraModelManipulator.remove(container, feature, index);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  protected void doChangeIndex(final EObject container, final EStructuralFeature feature, final Object element, final int index) {
    try {
      Object _eGet = container.eGet(feature);
      final int oldIndex = ((EList) _eGet).indexOf(element);
      this.viatraModelManipulator.changeIndex(container, feature, oldIndex, index);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
