/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Balazs Grill, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.ui.utils;

import java.util.Arrays;
import org.eclipse.emf.diffmerge.patch.AttributeEntry;
import org.eclipse.emf.diffmerge.patch.ElementEntry;
import org.eclipse.emf.diffmerge.patch.ReferenceEntry;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchEntry;
import org.eclipse.xtend2.lib.StringConcatenation;

@SuppressWarnings("all")
public class ModelPatchEntryHelper {
  protected String _stringRepresentation(final ElementEntry entry) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("ElementEntry");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Direction: ");
    String _name = entry.getDirection().name();
    _builder.append(_name, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* ID: ");
    String _identifier = entry.getContext().getIdentifier();
    _builder.append(_identifier, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* Type: ");
    String _identifier_1 = entry.getType().getIdentifier();
    _builder.append(_identifier_1, " ");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }
  
  protected String _stringRepresentation(final AttributeEntry entry) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("AttributeEntry");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Direction: ");
    String _name = entry.getDirection().name();
    _builder.append(_name, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* Attribute owner: ");
    String _identifier = entry.getContext().getIdentifier();
    _builder.append(_identifier, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* Attribute: ");
    String _identifier_1 = entry.getFeature().getIdentifier();
    _builder.append(_identifier_1, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* Value: ");
    String _value = entry.getValue();
    _builder.append(_value, " ");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }
  
  protected String _stringRepresentation(final ReferenceEntry entry) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("ReferenceEntry");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("* Direction: ");
    String _name = entry.getDirection().name();
    _builder.append(_name, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* Reference owner: ");
    String _identifier = entry.getContext().getIdentifier();
    _builder.append(_identifier, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* Reference: ");
    String _identifier_1 = entry.getFeature().getIdentifier();
    _builder.append(_identifier_1, " ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("* Target: ");
    String _identifier_2 = entry.getTarget().getIdentifier();
    _builder.append(_identifier_2, " ");
    _builder.newLineIfNotEmpty();
    return _builder.toString();
  }
  
  protected String _stringRepresentation(final ModelPatchEntry entry) {
    return entry.toString();
  }
  
  public String stringRepresentation(final ModelPatchEntry entry) {
    if (entry instanceof AttributeEntry) {
      return _stringRepresentation((AttributeEntry)entry);
    } else if (entry instanceof ReferenceEntry) {
      return _stringRepresentation((ReferenceEntry)entry);
    } else if (entry instanceof ElementEntry) {
      return _stringRepresentation((ElementEntry)entry);
    } else if (entry != null) {
      return _stringRepresentation(entry);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(entry).toString());
    }
  }
}
