/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Balazs Grill, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.ui.utils;

import java.util.Arrays;
import org.eclipse.emf.diffmerge.patch.api.ModelPatch;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchEntry;
import org.eclipse.emf.diffmerge.patch.ui.utils.DefaultEntryLabelProvider;
import org.eclipse.emf.diffmerge.patch.ui.utils.IEntryLabelProvider;
import org.eclipse.emf.diffmerge.patch.ui.utils.IdentifiableWrapper;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class ModelPatchLabelProvider implements ILabelProvider {
  private IEntryLabelProvider entryLabelProvider;
  
  public ModelPatchLabelProvider() {
    DefaultEntryLabelProvider _defaultEntryLabelProvider = new DefaultEntryLabelProvider();
    this.entryLabelProvider = _defaultEntryLabelProvider;
  }
  
  public ModelPatchLabelProvider(final IEntryLabelProvider entryLabelProvider) {
    this.entryLabelProvider = entryLabelProvider;
  }
  
  @Override
  public Image getImage(final Object element) {
    return null;
  }
  
  @Override
  public String getText(final Object element) {
    return this.getLabel(element);
  }
  
  @Override
  public void addListener(final ILabelProviderListener listener) {
  }
  
  @Override
  public void dispose() {
  }
  
  @Override
  public boolean isLabelProperty(final Object element, final String property) {
    return true;
  }
  
  @Override
  public void removeListener(final ILabelProviderListener listener) {
  }
  
  private String _getLabel(final ModelPatch patch) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("ModelPatch [");
    int _size = patch.getEntries().size();
    _builder.append(_size);
    _builder.append("]");
    return _builder.toString();
  }
  
  private String _getLabel(final ModelPatchEntry entry) {
    return this.entryLabelProvider.getLabel(entry);
  }
  
  private String _getLabel(final IdentifiableWrapper wrapper) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(wrapper.name);
    _builder.append(": ");
    _builder.append(wrapper.ident);
    return _builder.toString();
  }
  
  private String _getLabel(final Object object) {
    return object.toString();
  }
  
  public String correctCase(final String string) {
    return StringExtensions.toFirstUpper(string.toLowerCase());
  }
  
  private String getLabel(final Object patch) {
    if (patch instanceof ModelPatch) {
      return _getLabel((ModelPatch)patch);
    } else if (patch instanceof ModelPatchEntry) {
      return _getLabel((ModelPatchEntry)patch);
    } else if (patch instanceof IdentifiableWrapper) {
      return _getLabel((IdentifiableWrapper)patch);
    } else if (patch != null) {
      return _getLabel(patch);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(patch).toString());
    }
  }
}
