/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Peter Lunk (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.ui.wizards;

import com.google.common.base.Objects;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchException;
import org.eclipse.emf.diffmerge.patch.ui.utils.ModelpatchApplicationDTO;
import org.eclipse.emf.diffmerge.patch.ui.wizards.pages.ModelpatchEDMWizardPage;
import org.eclipse.emf.diffmerge.patch.ui.wizards.pages.ModelpatchFilteringWizardPage;
import org.eclipse.emf.diffmerge.patch.ui.wizards.pages.ModelpatchLoadingPatchWizardPage;
import org.eclipse.emf.diffmerge.patch.ui.wizards.pages.ModelpatchResultWizardPage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.xtext.xbase.lib.Exceptions;

@SuppressWarnings("all")
public class ModelpatchApplicationWizard extends Wizard {
  private ModelpatchApplicationDTO dto;
  
  private ModelpatchLoadingPatchWizardPage patchLoadingPage;
  
  private ModelpatchFilteringWizardPage patchModificationPage;
  
  private ModelpatchResultWizardPage resultPage;
  
  private ModelpatchEDMWizardPage edmPage;
  
  public ModelpatchApplicationWizard(final Resource originalModel) {
    this.setWindowTitle("Modelpatch Application");
    ModelpatchApplicationDTO _modelpatchApplicationDTO = new ModelpatchApplicationDTO(originalModel);
    this.dto = _modelpatchApplicationDTO;
    ModelpatchLoadingPatchWizardPage _modelpatchLoadingPatchWizardPage = new ModelpatchLoadingPatchWizardPage(this.dto);
    this.patchLoadingPage = _modelpatchLoadingPatchWizardPage;
    ModelpatchFilteringWizardPage _modelpatchFilteringWizardPage = new ModelpatchFilteringWizardPage(this.dto);
    this.patchModificationPage = _modelpatchFilteringWizardPage;
    ModelpatchResultWizardPage _modelpatchResultWizardPage = new ModelpatchResultWizardPage(this.dto);
    this.resultPage = _modelpatchResultWizardPage;
    ModelpatchEDMWizardPage _modelpatchEDMWizardPage = new ModelpatchEDMWizardPage(this.dto);
    this.edmPage = _modelpatchEDMWizardPage;
  }
  
  @Override
  public void addPages() {
    this.addPage(this.patchLoadingPage);
    this.addPage(this.patchModificationPage);
    this.addPage(this.resultPage);
    this.addPage(this.edmPage);
  }
  
  @Override
  public boolean canFinish() {
    return (Objects.equal(this.getContainer().getCurrentPage(), this.resultPage) || (Objects.equal(this.getContainer().getCurrentPage(), this.edmPage) && this.edmPage.isPageComplete()));
  }
  
  @Override
  public boolean performFinish() {
    IWizardPage _currentPage = this.getContainer().getCurrentPage();
    boolean _equals = Objects.equal(_currentPage, this.resultPage);
    if (_equals) {
      this.dto.saveModel();
      return true;
    } else {
      IWizardPage _currentPage_1 = this.getContainer().getCurrentPage();
      boolean _equals_1 = Objects.equal(_currentPage_1, this.edmPage);
      if (_equals_1) {
        return true;
      }
    }
    return false;
  }
  
  @Override
  public boolean performCancel() {
    boolean _xblockexpression = false;
    {
      IWizardPage _currentPage = this.getContainer().getCurrentPage();
      boolean _equals = Objects.equal(_currentPage, this.edmPage);
      if (_equals) {
        this.dto.undoEDMModifications();
      }
      _xblockexpression = super.performCancel();
    }
    return _xblockexpression;
  }
  
  @Override
  public IWizardPage getNextPage(final IWizardPage currentPage) {
    boolean _equals = Objects.equal(currentPage, this.patchLoadingPage);
    if (_equals) {
      try {
        this.dto.loadPatch(this.patchLoadingPage.getSelectedFilePath());
      } catch (final Throwable _t) {
        if (_t instanceof ModelPatchException) {
          final ModelPatchException e = (ModelPatchException)_t;
          e.printStackTrace();
        } else {
          throw Exceptions.sneakyThrow(_t);
        }
      }
    }
    return super.getNextPage(currentPage);
  }
}
